/*##############################################################################

nIP

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/
/** @file dns_sd.h
 *
 * DNS Services Discovery Extension.
 *
 * Provides means to register and unregister services.
 *
 */

#ifndef _DNS_SD_H_
 #define _DNS_SD_H_

#include "nip_init.h"
#include "nip_types.h"
#include "os_core.h"
#include "mem.h"
#include "app/mdns.h"

#define NIP_DNSSD_NO_SERVICE   0

#ifndef NIP_DNSSD_MAX_SERVICES
	#define NIP_DNSSD_MAX_SERVICES 1
#endif


#if   NIP_DNSSD_MAX_SERVIECS < 0xFF
	typedef uint8_t nip_dnssd_id_t;
#elif NIP_DNSSD_MAX_SERVICES < 0xFFFF
	typedef uint16_t nip_dnssd_id_t;
#else
	typedef uint32_t nip_dnssd_id_t;
#endif

struct nip_dnssd_name_srv
{
	nip_mdns_namestring_id_t instance;
	nip_mdns_namestring_id_t app;
	nip_mdns_namestring_id_t proto;
	nip_mdns_namestring_id_t domain;
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;

struct nip_dnssd_name_ptr
{
	nip_mdns_namestring_id_t app;
	nip_mdns_namestring_id_t proto;
	nip_mdns_namestring_id_t domain;
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;

struct nip_dnssd_name_a
{
	nip_mdns_namestring_id_t host;
	nip_mdns_namestring_id_t domain;
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;

struct nip_dnssd_name_txt
{
	nip_mdns_namestring_id_t instance;
	nip_mdns_namestring_id_t app;
	nip_mdns_namestring_id_t proto;
	nip_mdns_namestring_id_t domain;
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;

typedef union
{
	struct nip_mdns_name      generic;
	struct nip_dnssd_name_a   a;
	struct nip_dnssd_name_srv srv;
	struct nip_dnssd_name_ptr ptr;
	struct nip_dnssd_name_txt txt;
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
nip_dnssd_name_t;


/** Structure to hold IDs of registered DNS Records for a specific service. */
struct nip_dnssd_service
{
	nip_dnssd_id_t      id;
	nip_mdns_cache_id_t srv_rr; /**< SRV-record */
	nip_mdns_cache_id_t txt_rr; /**< TXT-record */
	nip_mdns_cache_id_t ptr_rr; /**< PTR-record */
	nip_mdns_cache_id_t dns_sd_ptr_rr; // _services._dns-sd._udp.local PTR
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;



nip_dnssd_id_t nip_dnssd_register( nip_mem_ptr_t *instance, nip_mem_ptr_t *service, nip_mem_ptr_t *txt, nip_mem_size_t txtlen, uint16_t port, uint16_t priority, uint16_t weight );

#endif /* _DNS_SD_H_ */
