/*##############################################################################

nIP - nano IP stack

File        : arp.h

Description : nIP Internetstack ARP implementation

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/

#ifndef _NIP_ARP_H
#define _NIP_ARP_H

#include "os_core.h"

#ifndef NIP_ARP_CACHE_SIZE
#define NIP_ARP_CACHE_SIZE  4   /**< number of entries in ARP cache */
#endif
#define NIP_ARP_CACHE_LIFE  300 /**< lifetime of ARP cache entry in seconds */
#define NIP_ARP_MAX_RETRIES 3   /**< maximum number of ARP resolve retries */
#define NIP_ARP_RETRY_INTVL 1   /**< interval of ARP resolve retries in seconds*/

#define NIP_ARP_OP_REQUEST 0x01
#define NIP_ARP_OP_REPLY   0x02

// #define NIP_ARP_HW_TYPE 0x0001
#define NIP_ARP_HW_SIZE 6             /**< ethernet address length */
#define NIP_ARP_PR_TYPE NIP_NUM_PR_IP /**< IP protocol type in network byte order */
#define NIP_ARP_PR_SIZE 4             /**< IP address length */

/** ARP packet structure.
 * @note That this implementation of ARP is limited to IP and Ethernet addresses
 * being used. For usage with other protocols the above ..SIZE defines have to
 * be adapted or for use with multiple protocols the implementation has to be
 * modified to support dynamically sized ARP packets.
 */
struct ARP_PACKET {
	uint16_t hrd;    /**< hardware address space, e.g. Ethernet:0x0001 */
	uint16_t pro;    /**< protocol address space, e.g. IP:0x0800 */
	uint8_t hln;     /**< byte length of each hardware address */
	uint8_t pln;     /**< byte length of each protocol address */
	uint8_t op[2];   /**< opcode, Request:0x0001, Reply:0x0002 */
	uint8_t sha[NIP_ARP_HW_SIZE]; /**< hardware address of sender */
	uint8_t spa[NIP_ARP_PR_SIZE]; /**< protocol address of sender */
	uint8_t tha[NIP_ARP_HW_SIZE]; /**< hardware address of target */
	uint8_t tpa[NIP_ARP_PR_SIZE]; /**< protocol address of target */
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;


#if ( NIP_ARP_CACHE_SIZE <= 256 )
typedef uint8_t  arp_cache_id_t;
#elif ( NIP_ARP_CACHE_SIZE <= 65536 )
typedef uint16_t arp_cache_id_t;
#else
typedef uint32_t arp_cache_id_t;
#endif


struct ARP_CACHE {
	uint8_t  hw_addr[NIP_ARP_HW_SIZE]; /**< hardware address */
	uint8_t  pr_addr[NIP_ARP_PR_SIZE]; /**< protocol address */
	nip_time_t timestamp;              /**< timeout time of this entry */
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
;

void nip_arp_disp_resolve_any( void );
void nip_disp_arp_receive( void );
void nip_arp_disp_resolve( void );
void nip_arp_disp_probe_announcement( void );
#endif /* _NIP_ARP_H */

