/*##############################################################################

nIP - nano IP stack

File        : numbers.h

Description : Assigned Numbers in Network Byte Order

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/


#ifndef _NUMBERS_H_
#define _NUMBERS_H_

// include definition of ENDIANNESS
#include NIP_ARCH_FILE(endianness.h)

// The predefined numbers will be stored depending on the system's byte
// order. That will spare future conversions between host and network byte order.

#if ( ENDIANNESS == BIG_ENDIAN )

/** \name Protocol Types for ARP and Ethernet*/
/** check "Assigned Numbers" RFC for more */
//@{
#define NIP_NUM_PR_IP               0x0800
#define NIP_NUM_PR_ARP              0x0806
#define NIP_NUM_PR_IPv6             0x86DD
#define NIP_NUM_IP_PROTO_UDP        0x0011 /* 16-bit representation of protocol ID 17*/
#define NIP_NUM_IP_PROTO_TCP        0x0006 /* 16-bit representation of protocol ID 06 */
//@}
/** \name other constants*/
//@{
#define NIP_NUM_DNS_CLASS_IN        0x0001 /* DNS record class IN, decimal value 0001 */
//@}

#else  // ENDIANNESS

/** \name Protocol Types for ARP and Ethernet*/
/** check "Assigned Numbers" RFC for more */
//@{
#define NIP_NUM_PR_IP               0x0008
#define NIP_NUM_PR_ARP              0x0608
#define NIP_NUM_PR_IPv6             0xDD86

#define NIP_NUM_IP_PROTO_UDP        0x1100 /* 16-bit representation of protocol ID 17 */
#define NIP_NUM_IP_PROTO_TCP        0x0600 /* 16-bit representation of protocol ID 06 */
//@}

/** \name other constants*/
//@{
#define NIP_NUM_DNS_CLASS_IN        0x0100 /* DNS record class IN, decimal value 0001 */
//@}

#endif // ENDIANNESS

#endif //_NUMBERS_H_
