/*##############################################################################

nIP - nano IP stack

File        : nip_error.h

Description : nIP Error Codes

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/

#ifndef _NIP_ERROR_H
 #define _NIP_ERROR_H

/** Errorcodes to be used by NIP functions. */
typedef enum
{
	NIP_E_OK,           //!< no error
	NIP_E_PARAM_FAILED, //!< function received invalid parameter
	NIP_E_INVALID_BLOCK,//!< function received invalid block id
	NIP_E_BLOCK_LOCKED, //!< tried to invalidly access locked memory block
	NIP_E_OUT_OF_MEMORY,//!< not enough dynamic memory available for allocation
	NIP_E_EOF,          //!< end of file or end block data reached
	NIP_E_RESOLVING_ADDR,//!< (still) resolving target address, don't send data yet
	NIP_E_HARD_ERROR,   //!< some unexpected hardware error occured
	NIP_E_DEV_BUSY,     //!< network interface busy, try again later
	NIP_E_DISP_RUNNING, //!< dispatcher already running
	NIP_E_INVALID_SOCK, //!< given socket id was invalid
	NIP_E_NO_ROUTE_TO_HOST,//!< found no route to target host
	NIP_E_HOST_NOT_FOUND,  //!< target host not found on local network
	NIP_E_AGAIN,        //!< try again later
	NIP_E_OUT_OF_RESSOURCES,
	NIP_E_LOCKED,       //!< ressource locked
	NIP_E_NOT_FOUND,    //!< ressource not found
	NIP_E_EXISTS,       //!< could not create ressource, because it exists already
	NIP_E_NOT_CONFIGURED,//!< ressource not configured properly
	NIP_E_INTERNAL_ERROR,//!< internal error occured, data integrity unknown
	NIP_E_NO_MATCH       //!< compared ressources don't match
}
#ifndef DOXYGEN
__attribute__((packed))
#endif
nip_error_t;

extern nip_error_t nip_error;

#endif /* _NIP_ERROR_H */
