/*##############################################################################

nIP - nano IP stack

File        : os_core.h

Description : Internet Stack Operating System functions

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

##############################################################################*/

#ifndef _NIP_OS_CORE_H
#define _NIP_OS_CORE_H

#include <string.h>
#include "nip_types.h"
#include "avr/interrupt.h"

typedef uint32_t nip_time_t;

extern uint32_t clock;
extern uint32_t tickcount;


/** get time of day
 *
 * @return the value of time in seconds since 0 hours, 0 minutes, 0 seconds,
 * January 1st, 1970, Coordinated Universal Time, without including leap seconds.
 */
// nip_time_t nip_time( void );
#define nip_time() clock

/** get system tickcount
 * @return seconds since system start. Use this value for timeout measurement as
 * it will always be increased linearly. */
#define nip_tickcount() tickcount

/** compare memory areas */
#define nip_memcmp(s1, s2, n) memcmp(s1, s2, n)

/** copy memory area */
void nip_memcpy( void* dest, void* src, uint16_t size);
//#define nip_memcpy(dest, src, n) memcpy(dest, src, n)
#define nip_memmove(dest, src, n) memmove(dest, src, n)

/** fill memory with constant byte */
// void nip_memset( void* ptr, uint8_t c, uint16_t size );
#define nip_memset(start, b, n) memset(start, b, n)

/** disable global interrupts */
#define nip_int_off() cli()

/** enable global interrupts */
#define nip_int_on() sei()


#endif /* _NIP_OS_CORE_H */
