/*##############################################################################

nIP - Testapplication

File        : httpd.c

Description : Simple Web Server

Copyright notice:

Copyright (C) 2005 -
Andreas Dittrich, dittrich@informatik.hu-berlin.de
Jon Kowal, kowal@informatik.hu-berlin.de

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#############################################################################*/

#include "httpd.h"
#include "dispatcher.h"
#include "inet.h"
#include "os_core.h"
#include "nip_error.h"
#include "net/net_if.h"
#include "net/tcp.h"
#include "net/numbers.h"
#include "app/ports.h"
#include "app/dns_sd.h"

#if HTTPD_ENABLE == 1

nip_tcp_sock_id_t        http_listensock;
nip_tcp_sock_id_t        http_opensock = NIP_TCP_NO_SOCKET;


void httpd_start( void )
{
	struct nip_tcp_sock_addr sockaddr;

	// open tcp socket
	nip_memset( &sockaddr, 0, sizeof( struct nip_tcp_sock_addr ) );
	sockaddr.port = NIP_TCP_PORT_HTTP;
	http_listensock = nip_tcp_socket( &sockaddr, NIP_TCP_SOCK_FLG_LISTENING );
}

void httpd_check( void )
{
	nip_mem_size_t      size;
	uint8_t             buf;
	uint8_t             p;
	static uint8_t      found_addr;
	static http_state_t state;
	static uint8_t      nl_pos; /* characters parsed since newline */
	static struct http_parser parser[HTTP_PARSER_COUNT] = {
	                                        {0, 6, (uint8_t*)"GET /A"  },
	                                        {0, 6, (uint8_t*)"GET /B"  },
	                                        {0, 2, (uint8_t*)"\r\n"}};
#if (NIP_DNSSD_ENABLE == 1)
	nip_dnssd_id_t srv_id = NIP_DNSSD_NO_SERVICE;
	uint8_t *instance_name = (uint8_t*)"\x07""Boogie!";
	uint8_t *service_name  = (uint8_t*)"\x05""_http""\x04""_tcp";
	uint8_t *txt_records   = (uint8_t*)"\x07""path=/B";
	nip_mem_ptr_t  in_mem_ptr;
	nip_mem_ptr_t  srv_mem_ptr;
	nip_mem_ptr_t  txt_mem_ptr;
#endif /*NIP_DNSSD_ENABLE*/

#if (NIP_DNSSD_ENABLE == 1)
	// register service
	if ( srv_id == NIP_DNSSD_NO_SERVICE )
	{
		in_mem_ptr.id  = NIP_MEM_NULL;
		in_mem_ptr.ptr = instance_name;
	// 			instance_name[0] = 3;
		srv_mem_ptr.id  = NIP_MEM_NULL;
		srv_mem_ptr.ptr = service_name;
	// 			service_name[0] = 7;
	// 			service_name[8] = 4;
		txt_mem_ptr.id  = NIP_MEM_NULL;
		txt_mem_ptr.ptr = txt_records;
	// 			txt_records[0]  = 8;
		srv_id = nip_dnssd_register( &in_mem_ptr, &srv_mem_ptr, &txt_mem_ptr,8, NIP_TCP_PORT_HTTP, 0, 0 );
	}
#endif /*NIP_DNSSD_ENABLE*/

	// check for new connections if we're not already serving one
	if ( http_opensock == NIP_TCP_NO_SOCKET )
	{
		http_opensock = nip_tcp_accept( http_listensock );
		if ( http_opensock != NIP_TCP_NO_SOCKET )
		{
			nl_pos        = 0;
			parser[0].pos = 0;
			parser[1].pos = 0;
			parser[2].pos = 0;
			state         = HTTP_STAT_ADDR;
		}
	}

	if ( http_opensock != NIP_TCP_NO_SOCKET )
	{
		// check for incoming data
		size = nip_tcp_read( http_opensock, &buf, 1 );

		// connection closed by remote peer?
		/// @todo include some kind of timeout after which stale connections
		/// are being reset so we won't be locked forever by broken http-clients.
		if ( size == 0 && nip_error != NIP_E_AGAIN )
		{
			// close connectin
			if ( state != HTTP_STAT_CLOSE )
				nip_tcp_close( http_opensock );

			http_opensock = NIP_TCP_NO_SOCKET;
		}
		// received data?
		else if ( size > 0 && state != HTTP_STAT_CLOSE )
		{
			// increment distance to previous newline
			nl_pos++;
			p = 0;
			do
			{
				///@todo check for header lines that are longer than 255 and would
				/// therefor cause counter overflows.
				// scan for header end
				if ( buf == parser[p].s[parser[p].pos] )
					parser[p].pos++;
				else
					parser[p].pos = 0;

				// check address
				if ( state == HTTP_STAT_ADDR
				  && p <= HTTP_ADDR_PARSER
				  && parser[p].pos == parser[p].sz )
				{
					state = HTTP_STAT_200;
					found_addr = p;
				}

				// parse for newlines
				if ( p == HTTP_NL_PARSER && parser[p].pos == 2 )
				{
					// address should have been on line 1
					if ( state == HTTP_STAT_ADDR )
						state = HTTP_STAT_404;

					if ( nl_pos == parser[p].pos )
						goto header_done;

					parser[p].pos = 0;
					nl_pos = 0;
				}

			}
			while ( ++p < HTTP_PARSER_COUNT );

		}
	}

	goto quit;

	header_done:

	/// @todo perform checks on nip_tcp_write() operations
	if ( state == HTTP_STAT_404 )
	{
		nip_tcp_write( http_opensock, (uint8_t*)"HTTP/1.0 404\r\ncontent-type:text/html\r\n\r\n", 40 );
		nip_tcp_write( http_opensock, (uint8_t*)"<h1>404 Not Found</h1>", 22 );
	}

	if ( state == HTTP_STAT_200 )
	{
		/// @todo add site processing here
		nip_tcp_write( http_opensock, (uint8_t*)"HTTP/1.0 200\r\ncontent-type:", 27 );
		if ( found_addr == 0 )
		{
			nip_tcp_write( http_opensock, (uint8_t*)"text/plain\r\n\r\n", 14);
			buf = 250;
			nip_tcp_write( http_opensock, &buf, 1 );
		}
		else if ( found_addr == 1 )
			nip_tcp_write( http_opensock, (uint8_t*)"text/html\r\n\r\n<h1>Ooh Yes!! Boogie!!</h1>", 40 );
	}

	nip_tcp_close( http_opensock );
	state = HTTP_STAT_CLOSE;

	quit:
		return;
}

#endif
